@@ -8,11 +8,13 @@ IMPORTANT:
 
 import math
 
+from photonvision import PhotonUtils
+
 from robotpy_toolkit_7407.sensors.photonvision.photon_target import PhotonTarget, AprilTag
 from robotpy_toolkit_7407.sensors.photonvision.photon_camera import PhotonCamera
 from robotpy_apriltag import AprilTagFieldLayout
 from robotpy_toolkit_7407.sensors.gyro import Gyro
-from wpimath.geometry import Pose3d, Translation3d, Rotation3d
+from wpimath.geometry import Pose3d, Translation3d, Rotation3d, Transform2d, Translation2d, Rotation2d, Transform3d
 
 
 def LoadFieldLayout(json_path: str):
@@ -20,7 +22,8 @@ def LoadFieldLayout(json_path: str):
 
 
 class PhotonOdometry:
-    def __init__(self, camera: PhotonCamera, field_layout: dict, gyro: Gyro, start_pose=Pose3d(Translation3d(0, 0, 0), Rotation3d(roll=0, pitch=0, yaw=0))):
+    def __init__(self, camera: PhotonCamera, field_layout: dict, gyro: Gyro,
+                 start_pose=Pose3d(Translation3d(0, 0, 0), Rotation3d(roll=0, pitch=0, yaw=0))):
         self.camera = camera
         self.field_layout = self.parse_field_layout(field_layout)
         self.gyro = gyro
@@ -31,6 +34,7 @@ class PhotonOdometry:
         self.pose_estimate = self.getRobotPose()
 
     def getRobotPose(self, target: PhotonTarget = None):
+
         if target is None:
             target = self.camera.latest_best_target
 
@@ -42,6 +46,28 @@ class PhotonOdometry:
         except:
             return None
 
+        camera_to_target = target.relative_pose
+        camera_to_target_TRANSFORM = Transform2d(
+            camera_to_target.translation().toTranslation2d(),
+            camera_to_target.rotation().toRotation2d()
+        )
+
+        camera_to_robot = self.camera.camera_to_robot_pose
+        camera_to_robot_TRANSFORM = Transform2d(
+            camera_to_robot.translation().toTranslation2d(),
+            camera_to_target.rotation().toRotation2d()
+        )
+
+        field_to_target = self.field_layout.getTagPose(target.ID).toPose2d()
+
+        estimated_field_to_robot_pose = PhotonUtils.estimateFieldToRobot(
+            cameraToTarget=camera_to_target_TRANSFORM,
+            cameraToRobot=camera_to_robot_TRANSFORM,
+            fieldToTarget=field_to_target,
+        )
+
+        print("ESTIMATED FIELD TO ROBOT POSE: ", estimated_field_to_robot_pose)
+
         gyro_angle = self.gyro.get_robot_heading()
         camera_to_target = target.relative_pose
         robot_to_camera = self.camera.camera_to_robot_pose
@@ -56,7 +82,7 @@ class PhotonOdometry:
 
         field_to_robot = (
             field_to_camera[0] - h * math.sin(theta),
-            field_to_camera[1] + h * math.cos(theta)
+            field_to_camera[1] - h * math.cos(theta)  # Testing positive did not work
         )
 
         return field_to_robot
